@@ -3,7 +3,7 @@ import os
 import posixpath
 
 import StringIO
-from fabric.operations import run, local
+from fabric.operations import run, local, put
 from fabric.api import task, env, get
 from fabric.contrib.files import exists
 from fabric.state import _AttributeDict
@@ -34,6 +34,9 @@ class Configuration(object):
         },
         'gunicorn': {
             'workers': 2,
+            'maxrequests': 0,
+            'timeout': None,
+            'bind': None,
         },
         'celery': {
             'enabled': False,
@@ -52,6 +55,10 @@ class Configuration(object):
             'appendonly': 'no',
             'template': app_package_path('templates/redis/redis.conf'),
         },
+        'memcached': {
+            'enabled': True,
+            'size': 64,
+        },
         'processes': {
 
         },
@@ -62,22 +69,28 @@ class Configuration(object):
 
         },
         'nginx': {
+            'enabled': True,
             'client_max_body_size': '10m',
             'template': app_package_path('templates/nginx/nginx.conf'),
         },
         'supervisor': {
             'template': app_package_path('templates/supervisor/supervisor.conf'),
+            'daemon_template': app_package_path('templates/supervisor/supervisord.conf'),
             'group_template': app_package_path('templates/supervisor/supervisor-group.conf'),
             'gunicorn_command_template': app_package_path('templates/supervisor/gunicorn_command'),
             'celeryd_command_template': app_package_path('templates/supervisor/celeryd_command'),
             'celerycam_command_template': app_package_path('templates/supervisor/celerycam_command'),
+            'supervisorctl_command': None,
+            'supervisord_command': None,
+            'use_global_supervisord': False,
         },
         'newrelic': {
             'enabled': False,
             'config_file': 'newrelic.ini',
             'environment_name': '',
             'license': '',
-        }
+        },
+        'logdir': None,
     }
     def load_sites(self, config_file_content=None, env_dict=None):
         """
@@ -107,7 +120,7 @@ class Configuration(object):
         for site in config.section_namespaces("django") or ["main"]:
             attr_dict = self.defaults.copy()
             for key, value in attr_dict.items():
-                attr_dict[key] = _AttributeDict(value.copy())
+                attr_dict[key] = None if value is None else _AttributeDict(value.copy())
             for section in config.sections():
                 section = section.split(":")[0]
                 if self.defaults.get(section) is None:
@@ -200,9 +213,18 @@ class Configuration(object):
             "django_cmd": site_dict.django['cmd'],
             "django_args": " ".join(site_dict.get("django").get("args", [])),
         }
+        socket = posixpath.normpath(posixpath.join(env_dict.get("path"), "..", "tmp", "%s_%s_gunicorn.sock" % (env_dict.get("user"), site)))  # Asserts pony project layout
+        if site_dict.gunicorn['bind']:
+            bind = site_dict.gunicorn['bind']
+        else:
+            bind = 'socket:{}'.format(socket)
         gunicorn_cmd_context = {
-            "socket": posixpath.normpath(posixpath.join(env_dict.get("path"), "..", "tmp", "%s_%s_gunicorn.sock" % (env_dict.get("user"), site))), # Asserts pony project layout
+            "socket": socket,
+            "bind": bind,
             "workers": site_dict.gunicorn['workers'],
+            "maxrequests": site_dict.gunicorn['maxrequests'],
+            "timeout": site_dict.gunicorn['timeout'],
+
         }
         gunicorn_cmd_context.update(common_cmd_context)
         gunicorn_command_template_path = self.sites[site]['supervisor']['gunicorn_command_template']
@@ -215,27 +237,29 @@ class Configuration(object):
                     'priority': 100,
                 }
 
-        memcached_socket = posixpath.normpath(posixpath.join(env_dict.get("path"), "..", "tmp", "%s_%s_memcached.sock" % (env_dict.get("user"), site))) # Asserts pony project layout
-
-        process_dict["%s_%s_memcached" % (env_dict.get("user"), site)] = {
-                    'command': "memcached -s %s" % memcached_socket,
-                    'port': None,
-                    'socket': memcached_socket,
-                    'type': 'memcached',
-                    'priority': 60,
+        if site_dict.get("memcached").get("enabled"):
+            memcached_socket = posixpath.normpath(posixpath.join(env_dict.get("path"), "..", "tmp", "%s_%s_memcached.sock" % (env_dict.get("user"), site))) # Asserts pony project layout
+            process_dict["%s_%s_memcached" % (env_dict.get("user"), site)] = {
+                        'command': "memcached -s %s -m %d" % (memcached_socket, int(site_dict.get("memcached").get("size"))),
+                        'port': None,
+                        'socket': memcached_socket,
+                        'type': 'memcached',
+                        'priority': 60,
                 }
         if site_dict.get("celery").get("enabled"):
+            conf = site_dict.get("celery")
             cmd = env_dict.get("path") if not site_dict.get("newrelic").get("enabled") else '%sbin/newrelic-admin run-program %s' % (env_dict.get("path"), env_dict.get("path"))
             cmd += 'bin/celery'
             celeryd_command_context = {
-                'concurrency': site_dict.get("celery").get("concurrency"),
-                'maxtasksperchild': site_dict.get("celery").get("maxtasksperchild"),
-                'loglevel': site_dict.get("celery").get("loglevel"),
+                'concurrency': conf.get("concurrency"),
+                'maxtasksperchild': conf.get("maxtasksperchild"),
+                'loglevel': conf.get("loglevel"),
                 'path': env_dict.get("path"),
-                'version': site_dict.get("celery").get("version"),
-                'celery_app': site_dict.get("celery").get("app"),
-                'has_cam': site_dict.get("celery").get("celerycam"),
+                'version': conf.get("version"),
+                'celery_app': conf.get("app"),
+                'has_cam': conf.get("celerycam"),
                 'cmd': cmd,
+                'pidfile': posixpath.normpath(posixpath.join(env_dict.get("path"), '..', 'tmp', 'celery-%s.pid' % site)),
             }
             celeryd_command_context.update(common_cmd_context)
             celeryd_command_template_path = self.sites[site]['supervisor']['celeryd_command_template']
@@ -246,13 +270,16 @@ class Configuration(object):
                     'socket': None,
                     'type': 'celeryd',
                     'priority': 40,
+                    'stopasgroup': 'true',
+                    'killasgroup': 'true',
+                    'stopwaitsecs': conf.get('stopwaitsecs', None),
                 }
-            if site_dict.get("celery").get("celerycam"):
+            if conf.get("celerycam"):
                 celerycam_command_context = {
-                    'loglevel': site_dict.get("celery").get("loglevel"),
+                    'loglevel': conf.get("loglevel"),
                     'path': env_dict.get("path"),
-                    'version': site_dict.get("celery").get("version"),
-                    'celery_app': site_dict.get("celery").get("app"),
+                    'version': conf.get("version"),
+                    'celery_app': conf.get("app"),
                     'cmd': cmd,
                 }
                 celerycam_command_context.update(common_cmd_context)
@@ -326,6 +353,8 @@ class Configuration(object):
             'big_body_endpoints': env_dict.get('big_body_endpoints', []),
             'home': '/home/%s' %  env_dict.get("user"),
         }
+        deployment_dict['logdir'] = env_dict.get("logdir") or os.path.join(deployment_dict['home'], 'log')
+
 
         if not env_dict.get("databases"):
             deployment_dict["databases"] = {
@@ -367,6 +396,9 @@ class Configuration(object):
         #################
         gunicorn_dict = self.sites[site].get("gunicorn")
         gunicorn_dict["workers"] = env_dict.get("gunicorn", {}).get("workers", gunicorn_dict.get("workers"))
+        gunicorn_dict["maxrequests"] = env_dict.get("gunicorn", {}).get("maxrequests", gunicorn_dict.get("maxrequests"))
+        gunicorn_dict["timeout"] = env_dict.get("gunicorn", {}).get("timeout", gunicorn_dict.get("timeout"))
+        gunicorn_dict["bind"] = env_dict.get("gunicorn", {}).get("bind", gunicorn_dict.get("bind"))
 
         ###############
         # Celery dict #
@@ -381,6 +413,7 @@ class Configuration(object):
         ##############
 
         nginx_dict = self.sites[site].get("nginx")
+        nginx_dict["enabled"] = env_dict.get("nginx", {}).get("enabled", nginx_dict.get("enabled"))
         nginx_dict["location_settings"] = {
             "client_max_body_size": env_dict.get("nginx", {}).get("client_max_body_size", nginx_dict.get("client_max_body_size")),
         }
@@ -393,16 +426,36 @@ class Configuration(object):
         redis_dict = self.sites[site].get("redis")
         redis_dict["template"] = env_dict.get("redis", {}).get("template", redis_dict.get("template"))
 
+        ##################
+        # memcached dict #
+        ##################
+
+        memcached_dict = self.sites[site].get("memcached")
+        memcached_dict["enabled"] = env_dict.get("memcached", {}).get("enabled", memcached_dict.get("enabled"))
+        memcached_dict["size"] = env_dict.get("memcached", {}).get("size", memcached_dict.get("size"))
+
         ###################
         # supervisor dict #
         ###################
 
         supervisor_dict = self.sites[site].get("supervisor")
         supervisor_dict["template"] = env_dict.get("supervisor", {}).get("template", supervisor_dict.get("template"))
+        supervisor_dict["daemon_template"] = env_dict.get("supervisor", {}).get("daemon_template", supervisor_dict.get("daemon_template"))
         supervisor_dict["group_template"] = env_dict.get("supervisor", {}).get("group_template", supervisor_dict.get("group_template"))
         supervisor_dict["gunicorn_command_template"] = env_dict.get("supervisor", {}).get("gunicorn_command_template", supervisor_dict.get("gunicorn_command_template"))
         supervisor_dict["celeryd_command_template"] = env_dict.get("supervisor", {}).get("celeryd_command_template", supervisor_dict.get("celeryd_command_template"))
-        supervisor_dict["celerycam_command_template"] = env_dict.get("supervisor", {}).get("celerycam_command_template", supervisor_dict.get("celerycam_command_template"))
+        supervisor_dict["celeryd_command_template"] = env_dict.get("supervisor", {}).get("celeryd_command_template", supervisor_dict.get("celeryd_command_template"))
+        supervisor_dict["supervisorctl_command"] = env_dict.get("supervisor", {}).get("supervisorctl_command", supervisor_dict.get("supervisorctl_command"))
+        supervisor_dict["supervisord_command"] = env_dict.get("supervisor", {}).get("supervisord_command", supervisor_dict.get("supervisord_command"))
+        supervisor_dict["use_global_supervisord"] = env_dict.get("supervisor", {}).get("use_global_supervisord", supervisor_dict.get("use_global_supervisord"))
+        if supervisor_dict["supervisorctl_command"] is None:
+            if supervisor_dict["use_global_supervisord"]:
+                supervisor_dict["supervisorctl_command"] = 'sudo supervisorctl'
+            else:
+                supervisor_dict["supervisorctl_command"] = 'supervisorctl --config={}../config/supervisord.conf'.format(deployment_dict['path'])
+
+        if supervisor_dict["supervisord_command"] is None and not supervisor_dict["use_global_supervisord"]:
+            supervisor_dict["supervisord_command"] = 'supervisord -c {}../config/supervisord.conf'.format(deployment_dict['path'])
 
         #################
         # newrelic dict #
@@ -432,6 +485,7 @@ class Configuration(object):
             'celery': celery_dict,
             'nginx': nginx_dict,
             'redis': redis_dict,
+            'memcached': memcached_dict,
             'supervisor': supervisor_dict,
             'newrelic': newrelic_dict,
         }
@@ -502,3 +556,17 @@ def upload_media(from_dir="./tmp/media/", to_dir="../upload/media/"):
 @task
 def use_local_config_ini():
     env.use_local_config_ini = True
+
+@task
+def safe_put(*args, **kwargs):
+    """
+    a version of put that makes sure the directory exists first.
+    :return:
+    """
+    if len(args) >= 2:
+        dst_path = args[1]
+    else:
+        dst_path = kwargs.get('remote_path', None)
+    if dst_path:
+        run('mkdir -p {}'.format(os.path.dirname(dst_path)))
+    return put(*args, **kwargs)
